<?php

namespace Drupal\create_referencing_content\Plugin\ExtraField\Display;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Render\Element;
use Drupal\extra_field_plus\Plugin\ExtraFieldPlusDisplayBase;

/**
 * Create Referencing Content button pseudo-field display.
 *
 * @ExtraFieldDisplay(
 *   id = "create_referencing_content_button",
 *   label = @Translation("Create Referencing Content button"),
 *   deriver = "Drupal\extra_field_configuration\Plugin\Derivative\ExtraFieldConfigurationDeriver",
 *   visible = false
 * )
 */
class CreateReferencingContentButton extends ExtraFieldPlusDisplayBase {

  /**
   * {@inheritdoc}
   */
  public function view(ContentEntityInterface $entity) {
    $settings = $this->getEntityExtraFieldSettings();
    // These prepopulated reference links can only be provided when configured.
    if (!($settings['target_field'] ?? FALSE)) {
      return;
    }

    list($entity_type_id, $bundle, $field_name) = explode('.', $settings['target_field'], 3);
    $short_field = strpos($field_name, 'field_') === 0 ? substr_replace($field_name, '', 0, 6) : $field_name;
    $url = base_path() . $entity_type_id . '/add/' . $bundle . '?' . $short_field . '=' . $entity->id();
    $build['button'] = [
      '#type' => 'component',
      '#component' => 'create_referencing_content:create-referencing-content-button',
      '#props' => [
        'url' => $url,
        'label' => $settings['label'],
        'tooltip' => $settings['tooltip'],
        'classes' => explode(' ', $settings['classes']),
      ],
    ];

    return $build;
  }


  /**
   * {@inheritdoc}
   */
  protected static function extraFieldSettingsForm(): array {
    $form = parent::extraFieldSettingsForm();

    // @TODO consider if we want to allow tokens, like for bundle, in label and tooltip.
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => t('Button label'),
      '#description' => t('The anchor text.'),
    ];

    $form['tooltip'] = [
      '#type' => 'textfield',
      '#title' => t('Tooltip'),
      '#description' => t('Text for the link tooltip, already translated.'),
    ];

    $form['classes'] = [
      '#type' => 'textfield',
      '#title' => t('Classes'),
      '#description' => t('Space-separated classes to add to the HTML anchor tag for styling with CSS.'),
    ];

    // The all-important target field gets added in getExtraFieldSettingsForm().

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public static function getExtraFieldSettingsForm(string $field_id, string $entity_type_id, string $bundle, string $view_mode = 'default'): array {
    $default_values = (array) static::getDefaultExtraFieldSettings();
    $elements = (array) static::extraFieldSettingsForm();

    if (!empty($elements)) {
      foreach (Element::children($elements) as $name) {
        if (!isset($elements[$name]['#default_value']) && isset($default_values[$name])) {
          $elements[$name]['#default_value'] = $default_values[$name];
        }
      }
    }

    $all_fields_config = \Drupal::entityTypeManager()->getStorage('field_config')->loadMultiple();
    $reference_fields = array_filter($all_fields_config, function (FieldDefinitionInterface $field_def) use ($entity_type_id) {
      // Return field configurations where the field type is Entity Reference
      // with settings defined to target the current entity type (e.g. 'node')
      return (
        $field_def->getType() === "entity_reference"
        &&
        ($field_def->getSettings()['target_type'] ?? FALSE) === $entity_type_id
      );
    });

    $options = [];
    foreach ($reference_fields as $field_instance_id => $field_def) {
      $bundles = $field_def->getSettings()['handler_settings']['target_bundles'] ?? [];
      if (in_array($bundle, $bundles)) {
        // If we want to get the target entity bundle's display name, here's some part of how we could do that.
        // $content_type = \Drupal::entityTypeManager()->getStorage('node_type')->load($entity->bundle())->label();
        $options[$field_instance_id] = t("%field_name (%field_machine_name) in %bundle",
          [
            '%field_name' => $field_def->label(),
            '%field_machine_name' => $field_def->get('field_name'),
            // To get the bundle the field we do NOT use ->bundle() which gets
            // the configuration entity's bundle!  @TODO get the display name.
            '%bundle' => $field_def->get('bundle'),
          ]);
      }
    }

    $elements['target_field'] = [
      '#type' => 'select',
      '#title' => t('Target field'),
      '#options' => $options,
      '#required' => TRUE,
    ];

    return $elements;
  }



  /**
   * {@inheritdoc}
   */
  protected static function defaultExtraFieldSettings(): array {
    $values = parent::defaultExtraFieldSettings();

    $values += [
      'label' => '',
      'tooltip' => '',
      'classes' => 'button is-primary is-medium',
      'target_field' => FALSE,
    ];

    return $values;
  }

  /**
   * {@inheritdoc}
   */
  protected static function settingsSummary(string $field_id, string $entity_type_id, string $bundle, string $view_mode = 'default'): array {
    return [
      t('Link to @target_field with anchor text @label and tooltip @tooltip with classes @classes.', [
        '@target_field' => self::getExtraFieldSetting($field_id, 'target_field', $entity_type_id, $bundle, $view_mode),
        '@label' => self::getExtraFieldSetting($field_id, 'label', $entity_type_id, $bundle, $view_mode),
        '@tooltip' => self::getExtraFieldSetting($field_id, 'tooltip', $entity_type_id, $bundle, $view_mode),
        '@classes' => self::getExtraFieldSetting($field_id, 'classes', $entity_type_id, $bundle, $view_mode),
      ]),
    ];
  }

}
